/**********************************************************
Junia Howell
23 September 2016
Gini Index

Sections:
1) Setting up Program Syntax
2) Generating Segregation Variable
3) Calculating Total Segregation
4) Display Results

***********************************************************/

/**********************************************************
* Section 1: Setting up Program Syntax
***********************************************************/
* Part a- Opening Program
program gini, rclass byable(onecall) sortpreserve 

* Part b- Seting Version Control
version 12.0

* Part c- Setting Segregation Syntax
syntax varlist(numeric) [if] [in] ///
	[, GENerate(name) NODISplay weighted(varname numeric)]
marksample touse

quietly {

/**********************************************************
* Section 2: Generating Gini Unweighted
***********************************************************/
* Part a- Sorting By Ineqaulity Variable
if "`weighted'" == "" {
	sort `_byvars' `varlist'
	tempvar count max cum tractgini ginisim tot begin hold
	generate `count' = _n if `varlist' != . & `touse'
	bysort `_byvars': egen `begin' = min(`count') if `touse'
	replace `count' = `count' - `begin' + 1

	* Part b- Calculating Cummunative
	bysort `_byvars': egen double `max' = count(`count') if `touse'
	generate double `hold' = `varlist' if `count' == 1 & `touse'
	bysort `_byvars': egen double `cum' = min(`hold') if `touse'
	bysort `_byvars': replace `cum' = `varlist'[_n] + `cum'[_n-1] if _n>1 & `touse'

	* Part c- Calculating Gini for Tract
	generate double `tractgini' = (`max' + 1 - `count')*`varlist' if `touse'

	* Part d- Calculating Total Gini
	bysort `_byvars': egen double `ginisim' = total(`tractgini') if `touse'
	bysort `_byvars': egen double `tot' = max(`cum') if `touse'
	replace `ginisim' =  (1/`max')*((`max'+1)-(2*(`ginisim'/`tot'))) if `touse'
	}
/**********************************************************
* Section 3: Generating Gini Weighted
***********************************************************/
* Part a- Sorting By Ineqaulity Variable
else {
	sort `_byvars' `varlist'
	tempvar count max cum tractgini ginisim tot begin hold weighttotal cumtot precumtot
	generate `count' = _n if `varlist' != . & `touse'
	bysort `_byvars': egen `begin' = min(`count') if `touse'
	replace `count' = `count' - `begin' + 1
	
	* Part b- Calculating Cummunative
	bysort `_byvars': egen double `max' = count(`count') if `touse'
	bysort `_byvars': egen `weighttotal' = total(`weighted')

	generate double `hold' = `weighted'*`varlist' if `count' == 1 & `touse'

	bysort `_byvars': egen double `cum' = min(`hold') if `touse'
	bysort `_byvars': replace `cum' = `varlist'[_n]*`weighted'[_n] ///
		+ `cum'[_n-1] if _n>1 & `touse'

	
	replace `hold' = `weighted' if `count' == 1 & `touse'

	bysort `_byvars': egen double `cumtot' = min(`hold') if `touse'
	bysort `_byvars': replace `cumtot' = `weighted'[_n] ///
		+ `cumtot'[_n-1] if _n>1 & `touse'
	
	replace `hold' = 0 if `count' == 1 & `touse'

	bysort `_byvars': egen double `precumtot' = min(`hold') if `touse'
	bysort `_byvars': replace `precumtot' = `cumtot'[_n-1] if _n>1 & `touse'
				
	* Part c- Calculating Gini for Tract		
	generate double `tractgini' = ((`weighted'*(`weighttotal' + 1)) - ///
		(((`cumtot'*(`cumtot'+1))/2)-((`precumtot'*(`precumtot'+1))/2)))*`varlist' if `touse'

	bysort `_byvars': egen double `ginisim' = total(`tractgini') if `touse'
	bysort `_byvars': egen double `tot' = max(`cum') if `touse'
	replace `ginisim' =  (1/`weighttotal')*((`weighttotal'+1)- ///
		(2*(`ginisim'/`tot'))) if `touse'
	}
	
/**********************************************************
* Section 4: Display Results
***********************************************************/
* Part a- Saving Results
return scalar gini = `ginisim'

* Part b- Display IF there is not the no display option
if "`generate'" != "" {
	capture confirm variable `generate' 
		if !_rc {
            replace `generate' = `ginisim' if `touse'           
			}
		else {
            generate `generate' = `ginisim' if `touse' 
               }
}
} 
if "`nodisplay'" == "" {
	display "Gini of " "`varname'" " is " `ginisim'
}


end

exit
